function chart_ingreso_egreso (widget) {
    "use strict";

    var model = openerp;

    widget.ChartIngresoEgresoWidget = widget.Base.extend({
        template: 'ChartIngresoEgreso',
        data: [],

        events: {
            'click .week': 'showWeeks',
            'click .days': 'showDays',
            'click .month': 'showMonths',
        },

        init: function (parent) {
            this._super(parent, {
                width: 6,
                height: 4
            });
        },
        
        start: function () {
            var self = this;
            self.fetchInitial();
        },

        fetchInitial:function() {
            var self = this;
            self.fetchResUser().then(function (ResUser) {
                return ResUser;
            }).then(function (ResUser) {
                self.ResUser = ResUser;
                return self.fetchAccountJournal();
            }).then(function (AccountJournal) {
                self.AccountJournal = AccountJournal;
                return self.fetchResCompany();
            }).then(function(ResCompany) {
                self.ResCompany = ResCompany;
                return self.fetchResCurrency();
            }).then(function(ResCurrency) {
                self.ResCurrency = ResCurrency;
                return self.fecthAccountBankStatementLine();
            }).then(function (AccountBankStatementLine) {
                self.AccountBankStatementLine = AccountBankStatementLine;
                return self.showMonths();
            });
        },

        fetchResUser: function() {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','store_id'];
            var domain = [['id','=',self.session.uid]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountJournal: function() {
            var self = this;
            var defer = $.Deferred();
            var store_ids = self.ResUser[0].store_id[0];
            var fields = ['id', 'name', 'store_ids','currency'];
            var domain = [['type','in',['bank','cash']],['store_ids','in',store_ids]];
            var AccountJournal = new model.web.Model('account.journal');
            AccountJournal.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fecthAccountBankStatementLine: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(self.AccountJournal, function (item) {
                return item.id;
            }));
            var fields = ['id','date','amount','journal_id'];
            var domain = [['journal_id','in',journal_ids]];
            var AccountBankStatementLine = new model.web.Model('account.bank.statement.line');
            AccountBankStatementLine.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchResCompany: function() {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'currency_id'];
            var domain = [['id', '=', self.session.company_id]];
            var ResCompany = new model.web.Model('res.company');
            ResCompany.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchResCurrency : function(){
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'symbol', 'rate_silent', 'base', 'decimal_separator', 'decimal_places', 'thousands_separator', 'symbol_position'];
            var domain = [['active', '=', true]];
            var ResCurrency = new model.web.Model('res.currency');
            ResCurrency.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        getResCurrency: function (id) {
            var self = this;
            return _.filter(self.ResCurrency,function (item) {
                return item.id === id;
            })
        },

        getAccountJournal: function (id) {
            var self = this;
            return _.filter(self.AccountJournal,function (item) {
                return item.id === id;
            })
        },

        /*================================== 
            GET STATEMENTS LINES BY MONTHS
        ==================================*/
        
        getInAccountBankStatementLine:function(mes) {
            var self = this;
            if (mes < 10){
                var fecha = moment().format('YYYY')+'-'+'0'+mes;    
            }else{
                var fecha = moment().format('YYYY')+'-'+mes;    
            }
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).format('YYYY-MM') === fecha & inv.amount > 0;
            }));
        },

        getOutAccountBankStatementLine:function(mes) {
            var self = this;
            if (mes < 10){
                var fecha = moment().format('YYYY')+'-'+'0'+mes;    
            }else{
                var fecha = moment().format('YYYY')+'-'+mes;    
            }
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).format('YYYY-MM') === fecha & inv.amount < 0;
            }));
        },

        /*============================================ 
            GET STATEMENTS LINES BY THE CURRENT WEEK
        ============================================*/

        getInWeekAccountBankStatementLine:function(day) {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).week() === week & moment(inv.date).isoWeekday() === day & inv.amount > 0;
            }));
        },

        getOutWeekAccountBankStatementLine:function(day) {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).week() === week & moment(inv.date).isoWeekday() === day & inv.amount < 0;
            }));
        },

        /*================================ 
            GET STATEMENTS LINES BY DAYS
        ================================*/

        getInDaysAccountBankStatementLine:function(day) {
            var self = this;
            var date = moment().subtract(day, 'days').format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).format('YYYY-MM-DD') === date & inv.amount > 0;
            }));
        },

        getOutDaysAccountBankStatementLine:function(day) {
            var self = this;
            var date = moment().subtract(day, 'days').format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).format('YYYY-MM-DD') === date & inv.amount < 0; 
            }));
        },

        /*=============== 
            SHOW MONTHS
        ===============*/
        showMonths: function() {
            var self = this;
            var title = ['Ene', 'Feb', 'Mar','Abr','May','Jun','Jul','Ago','Sep','Oct','Nov','Dic'];
            var inStatement;
            var OutStatement;
            var mes = 0;
            var dataIn = [];
            var dataOut = [];
            var data = [];
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            for (var i = 1; i <= 12; i++) {
                inStatement = self.getInAccountBankStatementLine(i);
                var in_amount = 0;
                if(inStatement){
                    data = [];
                    _.each(inStatement, function (item) {
                        var journal = self.getAccountJournal(item.journal_id[0]).shift();
                        var currency;
                        if(journal.currency){
                            currency = self.getResCurrency(journal.currency[0]).shift();    
                        }else{
                            currency = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
                        }
                        data.push({
                            amount: item.amount * (CurrencyBase.rate_silent / currency.rate_silent),
                        })
                    });
                    var in_amount = _.reduce(_.map(data,function(item) {
                        return item.amount;
                    }),function(memo, num) {
                        return memo + num;
                    },0);
                } 
                dataIn.push(in_amount);
            }
            data = [];
            for (var i = 1; i <= 12; i++) {
                OutStatement = self.getOutAccountBankStatementLine(i);
                var out_amount = 0;
                if(OutStatement){
                    data = [];
                    _.each(OutStatement, function (item) {
                        var journal = self.getAccountJournal(item.journal_id[0]).shift();
                        var currency;
                        if(journal.currency){
                            currency = self.getResCurrency(journal.currency[0]).shift();    
                        }else{
                            currency = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
                        }
                        data.push({
                            amount: item.amount * (CurrencyBase.rate_silent / currency.rate_silent),
                        })
                    });
                    var out_amount = _.reduce(_.map(data,function(item) {
                        return item.amount;
                    }),function(memo, num) {
                        return memo + num;
                    },0);
                } 
                dataOut.push(Math.abs(out_amount));
            }
            self.fetchChart(dataIn, dataOut, title);
        },

        /*============= 
            SHOW WEEK
        =============*/
        showWeeks: function() {
            var self = this;
            var title = ['Lunes', 'Martes', 'Miercoles','Jueves','Viernes','Sabado','Domingo'];
            var inStatement;
            var OutStatement;
            var dataIn = [];
            var dataOut = [];
            var data = [];
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            for (var i = 1; i <= 7; i++) {
                inStatement = self.getInWeekAccountBankStatementLine(i);
                var in_amount = 0;
                if(inStatement){
                    data = [];
                    _.each(inStatement, function (item) {
                        var journal = self.getAccountJournal(item.journal_id[0]).shift();
                        var currency;
                        if(journal.currency){
                            currency = self.getResCurrency(journal.currency[0]).shift();    
                        }else{
                            currency = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
                        }
                        data.push({
                            amount: item.amount * (CurrencyBase.rate_silent / currency.rate_silent),
                        })
                    });
                    var in_amount = _.reduce(_.map(data,function(item) {
                        return item.amount;
                    }),function(memo, num) {
                        return memo + num;
                    },0);
                } 
                dataIn.push(in_amount);
            }
            for (var i = 1; i <= 7; i++) {
                OutStatement = self.getOutWeekAccountBankStatementLine(i);
                var out_amount = 0;
                if(OutStatement){
                    data = [];
                    _.each(OutStatement, function (item) {
                        var journal = self.getAccountJournal(item.journal_id[0]).shift();
                        var currency;
                        if(journal.currency){
                            currency = self.getResCurrency(journal.currency[0]).shift();    
                        }else{
                            currency = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
                        }
                        data.push({
                            amount: item.amount * (CurrencyBase.rate_silent / currency.rate_silent),
                        })
                    });
                    var out_amount = _.reduce(_.map(data,function(item) {
                        return item.amount;
                    }),function(memo, num) {
                        return memo + num;
                    },0);
                } 
                dataOut.push(Math.abs(out_amount));
            }
            self.fetchChart(dataIn, dataOut, title);
        },

        /*=====================
            SHOW LAST 15 DAYS
        =====================*/
        showDays: function() {
            var self = this;
            var title = [];
            var inStatement;
            var OutStatement;
            var dataIn = [];
            var dataOut = [];
            var data = [];
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            for (var i = 15; i >= 0; i--) {
                inStatement = self.getInDaysAccountBankStatementLine(i);
                var in_amount = 0;
                if(inStatement){
                    data = [];
                    _.each(inStatement, function (item) {
                        var journal = self.getAccountJournal(item.journal_id[0]).shift();
                        var currency;
                        if(journal.currency){
                            currency = self.getResCurrency(journal.currency[0]).shift();    
                        }else{
                            currency = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
                        }
                        data.push({
                            amount: item.amount * (CurrencyBase.rate_silent / currency.rate_silent),
                        })
                    });
                    var in_amount = _.reduce(_.map(data,function(item) {
                        return item.amount;
                    }),function(memo, num) {
                        return memo + num;
                    },0);
                } 
                dataIn.push(in_amount);
                title.push(moment().subtract(i, 'days').format('DD / MM'));
            }
            for (var i = 15; i >= 0; i--) {
                OutStatement = self.getOutDaysAccountBankStatementLine(i);
                var out_amount = 0;
                if(OutStatement){
                    data = [];
                    _.each(OutStatement, function (item) {
                        var journal = self.getAccountJournal(item.journal_id[0]).shift();
                        var currency;
                        if(journal.currency){
                            currency = self.getResCurrency(journal.currency[0]).shift();    
                        }else{
                            currency = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
                        }
                        data.push({
                            amount: item.amount * (CurrencyBase.rate_silent / currency.rate_silent),
                        })
                    });
                    var out_amount = _.reduce(_.map(data,function(item) {
                        return item.amount;
                    }),function(memo, num) {
                        return memo + num;
                    },0);
                } 
                dataOut.push(Math.abs(out_amount));
            }
            self.fetchChart(dataIn, dataOut, title);
        },

        /*=================
            BUILD A CHART
        =================*/
        fetchChart: function (dataIn,dataOut,title) {
            var self = this;
            var label = title;
            var bodyIn = dataIn;
            var bodyOut = dataOut;
            var item;

            Chart.scaleService.updateScaleDefaults('linear', {
              ticks: {
                callback: function(tick) {
                    return tick.toLocaleString('de-DE');
                }
              }
            }); 
            Chart.defaults.global.tooltips.callbacks.label = function(tooltipItem, data) {
                var dataset = data.datasets[tooltipItem.datasetIndex];
                var datasetLabel = dataset.label || '';
                return datasetLabel +  dataset.data[tooltipItem.index].toLocaleString('de-DE');
            };

            var chart = new Chart(this.$el.find(".widget-content").find('canvas'), {
                type: 'line',
                data: {
                    labels: label,
                    datasets: [
                        {
                            label: 'Ingreso  ',
                            data: bodyIn,
                            backgroundColor: 'rgba(51, 141, 255, 0.2)',
                            borderColor: '#0288d1',
                            borderWidth: 2,
                            fill: false,
                        },
                        {
                            label: 'Egreso  ',                       
                            data: bodyOut,
                            backgroundColor: 'rgba(255, 110, 59, 0.2)',
                            borderColor: '#ef6c00',
                            borderWidth: 2,
                            fill: false,
                        }
                    ]
                },
                options: {
                    responsive: true,
                    legend: {
                        display: false,
                    },
                    title: {
                        display: false,
                    },
                    layout: {
                        padding: {
                            top: 0,
                            bottom: 45,
                            left : 0,
                            rigth: 0,
                        }
                    },
                    events: ['click'],
                    elements: {
                        line: {
                            tension: 0.000001
                        },
                    },
                }
            });
        },
    });
}
